/*
 * Startup.S : processes after CPU RESET
 *   by uratan! 2012.10.9
 */

/*===================================================================*/

/*
 * Stack Configuration (Stack Sizes in Bytes)
 */
	.set Stack_Size, 0x00000100

.thumb
.section .stack, "w"
.align 4
	.space	(Stack_Size+3)&~3
Top_Stack:


/*===================================================================*/

.text

/*
 * Exception Vectors, Mapped to Address 0.
 *   VECTOR ADDRESSES SHALL BE ODD !  (there may be a directive for it...)
 */
.thumb
.section .vectorg, "ax"

	.extern	def_NMI_Handler
	.extern	def_HardFault_Handler
	.extern	def_Exception_Handler
	.extern	def_SVCall_Handler
	.extern	def_PendSV_Handler
	.extern	SysTick_Handler

__Vectors:
	.word	Top_Stack					/*  0: Initial SP     */
	.word	1+ Reset_Handler			/*  1: Reset          */
	.word	1+ def_NMI_Handler			/*  2: NMI            */
	.word	1+ def_HardFault_Handler	/*  3: HardFault      */
	.word	1+ def_Exception_Handler	/*  4: Reserved       */
	.word	1+ def_Exception_Handler	/*  5: Reserved       */
	.word	1+ def_Exception_Handler	/*  6: Reserved       */
	.word	0x00000000					/*  7: Reserved (boot signature) */
	.word	1+ def_Exception_Handler	/*  8: Reserved       */
	.word	1+ def_Exception_Handler	/*  9: Reserved       */
	.word	1+ def_Exception_Handler	/* 10: Reserved       */
	.word	1+ def_SVCall_Handler		/* 11: SVCall         */
	.word	1+ def_Exception_Handler	/* 12: Reserved       */
	.word	1+ def_Exception_Handler	/* 13: Reserved       */
	.word	1+ def_PendSV_Handler		/* 14: PendSV         */
	.word	1+ SysTick_Handler			/* 15: SysTick        */
		/* IRQs are continued... */
	.word	1+ def_Exception_Handler	/* 16: (0) SPI0_IRQ     */
	.word	1+ def_Exception_Handler	/* 17: (1) SPI1_IRQ     */
	.word	1+ def_Exception_Handler	/* 18: (2)  (rsrv)      */
	.word	1+ def_Exception_Handler	/* 19: (3) UART0_IRQ    */
	.word	1+ def_Exception_Handler	/* 20: (4) UART1_IRQ    */
	.word	1+ def_Exception_Handler	/* 21: (5) UART2_IRQ    */
	.word	1+ def_Exception_Handler	/* 22: (6)  (rsrv)      */
	.word	1+ def_Exception_Handler	/* 23: (7)  (rsrv)      */
	.word	1+ def_Exception_Handler	/* 24: (8) U2C0_IRQ     */
	.word	1+ def_Exception_Handler	/* 25: (9) SCT_IRQ      */
	.word	1+ def_Exception_Handler	/* 26:(10) MRT_IRQ      */
	.word	1+ def_Exception_Handler	/* 27:(11) CMP_IRQ      */
	.word	1+ def_Exception_Handler	/* 28:(12) WDT_IRQ      */
	.word	1+ def_Exception_Handler	/* 29:(13) BOD_IRQ      */
	.word	1+ def_Exception_Handler	/* 30:(14)  (rsrv)      */
	.word	1+ def_Exception_Handler	/* 31:(15) WKT_IRQ      */
	.word	1+ def_Exception_Handler	/* 32:(16)  (rsrv)      */
	.word	1+ def_Exception_Handler	/* 33:(17)  (rsrv)      */
	.word	1+ def_Exception_Handler	/* 34:(18)  (rsrv)      */
	.word	1+ def_Exception_Handler	/* 35:(19)  (rsrv)      */
	.word	1+ def_Exception_Handler	/* 36:(20)  (rsrv)      */
	.word	1+ def_Exception_Handler	/* 37:(21)  (rsrv)      */
	.word	1+ def_Exception_Handler	/* 38:(22)  (rsrv)      */
	.word	1+ def_Exception_Handler	/* 39:(23)  (rsrv)      */
	.word	1+ def_Exception_Handler	/* 40:(24) PININT0_IRQ  */
	.word	1+ def_Exception_Handler	/* 41:(25) PININT1_IRQ  */
	.word	1+ def_Exception_Handler	/* 42:(26) PININT2_IRQ  */
	.word	1+ def_Exception_Handler	/* 43:(27) PININT3_IRQ  */
	.word	1+ def_Exception_Handler	/* 44:(28) PININT4_IRQ  */
	.word	1+ def_Exception_Handler	/* 45:(29) PININT5_IRQ  */
	.word	1+ def_Exception_Handler	/* 46:(30) PININT6_IRQ  */
	.word	1+ def_Exception_Handler	/* 47:(31) PININT7_IRQ  */


/*===================================================================*/

/*
 * system startup processes
 */

.thumb
.section .init, "ax"
.global __startup
.func __startup
__startup:

Reset_Handler:

	/*
	 * Relocate .data section (Copy from ROM to RAM)
	 */
	LDR		R1, =_lma_data_start	/* src */
	LDR		R2, =_data_start		/* dest start */
	LDR		R3, =_data_end			/* dest end */
lp21:
	CMP		R2, R3
	BHS		lp22
	LDR		R4, [R1]
	STR		R4, [R2]
	ADD		R1, #4
	ADD		R2, #4
	B		lp21
lp22:

	/*
	 * Clear .bss section (Zero init)
	 */
	LDR		R2, =_bss_start
	LDR		R3, =_bss_end
	MOV		R4, #0
lp31:
	CMP		R2, R3
	BHS		lp32
	STR		R4, [R2]
	ADD		R2, #4
	B		lp31
lp32:

	/*
	 * don't forget to tune link.awk
	 */
#define COPY_and_EXECUTE_on_RAM  0

#if COPY_and_EXECUTE_on_RAM
	/*
	 * copy C related for RAM
	 */
	LDR		R1, =_lma_exec_start
	LDR		R2, =_exec_start
	LDR		R3, =_exec_end
lp51:
	CMP		R2, R3
	BHS		lp52
	LDR		R4, [R1]
	STR		R4, [R2]
	ADD		R1, #4
	ADD		R2, #4
	B		lp51
lp52:
#endif /* COPY_and_EXECUTE_on_RAM */

	/*
	 * monitor special Cortex-M0 registers
	 *   (do this after clearing bss)
	 */
#define LOG_VALUEs_of_SPECIAL_REGs  0

#if LOG_VALUEs_of_SPECIAL_REGs
	.extern get_specials
	bl	get_specials
#endif /* LOG_VALUEs_of_SPECIAL_REGs */

	/*
	 * Enter the C code
	 */
	LDR		R0,=main
	MOV		PC,R0

#if UARTm
	.word	2400			/* 2400bps */
	.word	0x0020			/* parity-even, 1stopbit */
	//.word	0x0000			/* parity-none, 1stopbit */
	//.word	0x0030			/* parity-odd, 1stopbit */
#endif /* UARTm */
.endfunc


.end
