#if UARTm
	/* not used */
#else /* UARTm */
/*
 * sctimer.c - generate 38kHz to modulate Ir output
 *   by uratan! 2014.10.24
 */
#include "common.h"
#include <chip.h>

/*
 * my functions
 */
void sctimer_c_init(u_long sysCLK);
void sct_enable_carrier(void);
void sct_disable_carrier(void);

u_long Tdiv;

/* =============================================================== */

/*
 * initial and run sctimer
 */
void sctimer_c_init(u_long sysCLK)
{
	u_long n, tmp;
	u_long Tdiv_half;

	/*
	 * enable clock for SCT
	 */
	LPC_SYSCTL->SYSAHBCLKCTRL |= (1 << SYSCTL_CLOCK_SCT);

	/*
	 * connect CTOUT_0 PIO0_3
	 */
	n = 6;
	tmp = LPC_SWM->PINASSIGN[n];
	tmp &= ~(0xff <<24);
	tmp |=  (  3  <<24);
	LPC_SWM->PINASSIGN[n] = tmp;

	/*
	 * setup SCT to generate 38kHz
	 *
	 *                  MAT[0]
	 *                      / -------- Tdiv
	 *                    / |EV0
	 *            MAT[1]/   | (clear)
	 *                /  -- | -------- Tdiv_half
	 *              / |EV1  |
	 *            /   |     |  /
	 *          /     V     |/
	 *       ---------------------> time
	 *          +-----+     +---
	 *   OUT0   |     |     |
	 *        --+     +-----+
	 */
	Tdiv = sysCLK / (38 * 1000);
	Tdiv_half = Tdiv / 2;
	Tdiv -= 1;				/* set (n-1) for 1/n */
	Tdiv_half -= 1;

	LPC_SCT->CONFIG = 0
					| (1<<0)		/* 32bit operation */
					| (0x00<<1)		/* use bus clock */
					| (1<<7)		/* no reload to match reegister */
					| (0<<17)		/* no limit function */
					| 0;
	LPC_SCT->CTRL_U = 0
					| (0<<0)		/* count-up mode (RO ?) */
					| (0<<1)		/* release stop */
					| (1<<2)		/* but keep halt */
					| (1<<3)		/* clear counter */
					| (0<<4)		/* count-up only */
					| (0x00<<5)		/* pre-scaler 1/1 */
					| 0;
	LPC_SCT->LIMIT = 0
				   | (0x01<<0)		/* EVENT0 limits the counter */
				   | 0;
	LPC_SCT->HALT = 0;				/* never halt */
	LPC_SCT->STOP = 0;				/* never stop */
	LPC_SCT->START = 0;				/* never re-start */
	LPC_SCT->COUNT_U = 0;			/* clear counter value */
	LPC_SCT->STATE = 0;				/* initial to STATE0 */
	LPC_SCT->REGMODE = 0;			/* use MAT[0] and MAT[1], no CAP[n] */
	//LPC_SCT->OUTPUT = 0;			/* no manual control */
	LPC_SCT->OUTPUTDIRCTRL = 0;		/* count-up only */
	LPC_SCT->RES = 0;				/* never conflict */

	LPC_SCT->EVEN = 0;				/* no use interrupt */
	//LPC_SCT->EVFLAG = 0x1f;
	LPC_SCT->CONEN = 0;				/* no use interrupt */
	//LPC_SCT->CONFLAG = 0x0f;

	LPC_SCT->MATCH[0].U = Tdiv;
	LPC_SCT->MATCH[1].U = Tdiv_half;
	LPC_SCT->MATCH[2].U = 0x0;
	LPC_SCT->MATCH[3].U = 0x0;
	LPC_SCT->MATCH[4].U = 0x0;
	LPC_SCT->MATCHREL[0].U = Tdiv;
	LPC_SCT->MATCHREL[1].U = Tdiv_half;
	LPC_SCT->MATCHREL[2].U = 0x0;
	LPC_SCT->MATCHREL[3].U = 0x0;
	LPC_SCT->MATCHREL[4].U = 0x0;

	LPC_SCT->EVENT[0].STATE = 0x01;		/* EVENT0 is valid at STATE0 */
	LPC_SCT->EVENT[0].CTRL = 0
						   | (0x00<<0)	/* select MAT[0] */
						   | (0x1<<12)	/* match only */
						   | 0;
	LPC_SCT->EVENT[1].STATE = 0x01;		/* EVENT1 is valid at STATE0 */
	LPC_SCT->EVENT[1].CTRL = 0
						   | (0x01<<0)	/* select MAT[1] */
						   | (0x1<<12)	/* match only */
						   | 0;
	LPC_SCT->EVENT[2].STATE = 0;	/* do not use EVENT2 */
	LPC_SCT->EVENT[2].CTRL = 0;		/* do not use EVENT2 */
	LPC_SCT->EVENT[3].STATE = 0;	/* do not use EVENT3 */
	LPC_SCT->EVENT[3].CTRL = 0;		/* do not use EVENT3 */
	LPC_SCT->EVENT[4].STATE = 0;	/* do not use EVENT4 */
	LPC_SCT->EVENT[4].CTRL = 0;		/* do not use EVENT4 */
	LPC_SCT->EVENT[5].STATE = 0;	/* do not use EVENT4 */
	LPC_SCT->EVENT[5].CTRL = 0;		/* do not use EVENT4 */

	LPC_SCT->OUT[0].SET = (1<<0);	/* EVENT0 sets OUT0 */
	LPC_SCT->OUT[0].CLR = (1<<1);	/* EVENT1 clrs OUT0 */
	LPC_SCT->OUT[1].SET = 0;		/* do not use OUT1 */
	LPC_SCT->OUT[1].CLR = 0;		/* do not use OUT1 */
	LPC_SCT->OUT[2].SET = 0;		/* do not use OUT2 */
	LPC_SCT->OUT[2].CLR = 0;		/* do not use OUT2 */
	LPC_SCT->OUT[3].SET = 0;		/* do not use OUT3 */
	LPC_SCT->OUT[3].CLR = 0;		/* do not use OUT3 */

	/*
	 * finally, disable pull-up-R on PIO0_3
	 */
	LPC_SYSCTL->SYSAHBCLKCTRL |= (1 << SYSCTL_CLOCK_IOCON);

	n = IOCON_PIO3;
	tmp = LPC_IOCON->PIO0[n];
	tmp &= ~PIN_MODE_MASK;
	tmp |= (PIN_MODE_INACTIVE <<PIN_MODE_BITNUM);
	LPC_IOCON->PIO0[n] = tmp;

	LPC_SYSCTL->SYSAHBCLKCTRL &= ~(1 << SYSCTL_CLOCK_IOCON);

//	/*
//	 * start the counter
//	 */
//	LPC_SCT->CTRL_U &= ~SCT_CTRL_HALT_L;	/* release halt */
}

/*
 *
 */
void sct_enable_carrier(void)
{
//	LPC_SCT->CTRL_U |= SCT_CTRL_CLRCTR_L;	/* clear counter */
	LPC_SCT->COUNT_U = Tdiv;			/* make first match just after start */
	LPC_SCT->CTRL_U &= ~SCT_CTRL_HALT_L;	/* release halt */
}

/*
 *
 */
void sct_disable_carrier(void)
{
	LPC_SCT->CTRL_U |= SCT_CTRL_HALT_L;		/* set halt */
	LPC_SCT->OUTPUT = 0;			/* make CTOUT_0 low */
}

/* =============================================================== */
#endif /* UARTm */
