/*
 * uart1.c - use UART1 to send IR signals
 *   by uratan! 2014.10.23
 */
#include "common.h"
#include <chip.h>

/*
 * my functions
 */
void uart1_c_init(u_long baudrate, u_long mainCLK);
#if UARTm
int u1_Keyin(void);
#else /* UARTm */
void u1_Putchar(int ch);
#endif /* UARTm */
#if 00000
u_char u1_Getchar(void);
void UART1_IRQ_Handler(void)  ATTR_INTR;
#endif /* 00000 */


/* =============================================================== */

/*
 * initial UART1
 */
void uart1_c_init(u_long baudrate, u_long mainCLK)
{
	u_long n, tmp;
	u_long Fdiv;

	/*
	 * enable clock for UART1
	 */
	LPC_SYSCTL->SYSAHBCLKCTRL |= (1 << SYSCTL_CLOCK_UART1);

	/*
	 * I expect that these are initialized with uart0
	 */
#if 0000
	/*
	 * enable clock for baudrate generator
	 */
	LPC_SYSCTL->UARTCLKDIV = 1;		/* mainCLK(12MHz or 60MHz) thru */

	/*
	 * setup fractional divider (1+5/8 = 13/8)
	 */
	LPC_SYSCTL->UARTFRGDIV  = 8*32 - 1;	/* use this (0xff only) */
	LPC_SYSCTL->UARTFRGMULT = 5*32;
#endif /* 0000 */

#if UARTm
	/*
	 * connect U1_RxD from PIO0_17 (PIO0_2 | PIO0_3), U1_TxD is not used
	 */
	n = 1;
	tmp = LPC_SWM->PINASSIGN[n];
	tmp &= ~(0xff <<16);
	tmp |=  ( 17  <<16);
	LPC_SWM->PINASSIGN[n] = tmp;
#else /* UARTm */
	/*
	 * connect U1_TxD for PIO0_2, U1_RxD is not used
	 */
	n = 1;
	tmp = LPC_SWM->PINASSIGN[n];
	tmp &= ~(0xff <<8);
	tmp |=  (  2  <<8);
	LPC_SWM->PINASSIGN[n] = tmp;
#endif /* UARTm */

	/*
	 * setup uart1
	 *   use Fractional Divider for 115kbps@12MHz (12MHz*8/13)
	 *       (it works well when mainCLK=60MHz)
	 */
#if UARTm
//	LPC_USART1->CFG = 0x0035;	/* enable UART, 8bit, odd-parity, 1stopbit */
	LPC_USART1->CFG = 0x0005 | *(u_long *)0x00ec;	/* enable UART, 8bit */
#else /* UARTm */
	LPC_USART1->CFG = 0x0005;	/* enable UART, 8bit, no-parity, 1stopbit */
#endif /* UARTm */
	LPC_USART1->CTRL = 0x000;	/* no special operations */
	LPC_USART1->INTENCLR = 0xf965;		/* do not use interrupts */
	Fdiv = 8 * mainCLK / 16 / baudrate / 13;	/* calc division */
	LPC_USART1->BRG = Fdiv - 1;
		/*
		 * 8bit BRG covers:
		 *     Fdiv  :  256    ...    1
		 *   ------------------------------
		 *   at 12MHz: 1803bps ...  461kbps
		 *   at 30MHz: 4507bps ... 1154kbps
		 */
#if UARTm
	/*
	 * finally, disable pull-up-R on port: PIO0_2, PIO_3
	 */
	LPC_SYSCTL->SYSAHBCLKCTRL |= (1 << SYSCTL_CLOCK_IOCON);

	n = IOCON_PIO2;
	tmp = LPC_IOCON->PIO0[n];
	tmp &= ~PIN_MODE_MASK;
	tmp |= (PIN_MODE_INACTIVE <<PIN_MODE_BITNUM);
	LPC_IOCON->PIO0[n] = tmp;

	n = IOCON_PIO3;
	tmp = LPC_IOCON->PIO0[n];
	tmp &= ~PIN_MODE_MASK;
	tmp |= (PIN_MODE_INACTIVE <<PIN_MODE_BITNUM);
	LPC_IOCON->PIO0[n] = tmp;

	LPC_SYSCTL->SYSAHBCLKCTRL &= ~(1 << SYSCTL_CLOCK_IOCON);
#else /* UARTm */
	/*
	 * finally, disable pull-up-R on output port (PIO0_2)
	 */
	LPC_SYSCTL->SYSAHBCLKCTRL |= (1 << SYSCTL_CLOCK_IOCON);

	n = IOCON_PIO2;
	tmp = LPC_IOCON->PIO0[n];
	tmp &= ~PIN_MODE_MASK;
	tmp |= (PIN_MODE_INACTIVE <<PIN_MODE_BITNUM);
	LPC_IOCON->PIO0[n] = tmp;

	LPC_SYSCTL->SYSAHBCLKCTRL &= ~(1 << SYSCTL_CLOCK_IOCON);
#endif /* UARTm */

#if 00000
	/*
	 * use interrupt to receive chars
	 */
	LPC_USART1->INTENSET = UART_INTEN_RXRDY;

	/*
	 * setup NVIC to enable interrupt
	 */
	NVIC_SetPriority(UART1_IRQn, PRI_UART1);
	NVIC_EnableIRQ(UART1_IRQn);
#endif /* 00000 */
}

#if UARTm
#else /* UARTm */
/*
 * write a char to UART1
 */
void u1_Putchar(int ch)
{
	/*
	 * wait for the buffer empty
	 */
	while( !(LPC_USART1->STAT & UART_STAT_TXRDY) ) {
		;
	}

	LPC_USART1->TXDATA = ch & 0xff;

#if 1
	/*
	 * wait for the the data sent
	 */
	while( !(LPC_USART1->STAT & UART_STAT_TXIDLE) ) {
		;
	}
#endif /* 1 */

	return;
}
#endif /* UARTm */

#if UARTm
/*
 * check arrival of a char from UART1
 */
int u1_Keyin(void)
{
	if(LPC_USART1->STAT & UART_STAT_RXRDY) {
#if UARTm
		return(LPC_USART1->RXDATA_STAT);		/* something has come */
					/* may want to see status with the data... */
#else /* UARTm */
		return(LPC_USART1->RXDATA & 0xff);		/* something has come */
#endif /* UARTm */
	}

	return(-1);		/* nothing has come */
}
#endif /* UARTm */

	/*
	 * do not use these functions
	 */
#if 00000
/*
 * wait until some char will come
 */
u_char u1_Getchar(void)
{
	int i;

	while(1) {
		i = cm_keyin();
		if(i != -1) {
			break;		/* something arrived */
		}
	}

	return(i);
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

/*
 * interrupt service routine
 */
void UART1_IRQ_Handler(void)
{
	if(LPC_USART1->STAT & UART_STAT_RXRDY) {
		rprintf(" %02x", LPC_USART1->RXDATA & 0xff);	/* something has come */
			/*
			 * UART_STAT_RXRDY flag is cleared by above read
			 */
	}
}
#endif /* 00000 */

/* =============================================================== */
